package pucrs.hslcommunicationservice.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;
import pucrs.hslcommunicationservice.aws.S3Client;
import pucrs.hslcommunicationservice.constants.DatabaseConstants;
import pucrs.hslcommunicationservice.domain.Announcement;
import pucrs.hslcommunicationservice.domain.Attachment;
import pucrs.hslcommunicationservice.repository.AnnouncementRepository;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class AnnouncementService {

    @Autowired
    private AnnouncementRepository announcementRepository;

    @Autowired
    private DatabaseConstants databaseConstants;

    @PersistenceContext
    private EntityManager entityManager;

    @Autowired
    private S3Client s3Client;

    public Announcement addAnnouncement(Announcement announcement) {
        List<Attachment> attachments = announcement.getAttachments()
                .parallelStream()
                .peek(attachment -> attachment.setFileName(generateUniqueAttachmentFilename(attachment.getFileName())))
                .peek(attachment -> s3Client.uploadAttachment(attachment.getFileName(), attachment.getContentAsBase64()))
                .collect(Collectors.toList());

        announcement.setAttachments(attachments);
        announcement.setDate(LocalDateTime.now());
        return announcementRepository.save(announcement);
    }

    public List<Announcement> getAnnouncements() {
        return getAnnouncements(null, null);
    }

    public List<Announcement> getAnnouncements(Integer page, Integer pageSize) {
        if (page == null) page = 0;
        if (pageSize == null) pageSize = databaseConstants.defaultPageSize;
        return announcementRepository.findAll(PageRequest.of(page, pageSize)).toList();
    }

    public List<Announcement> getAnnouncementsByDepartment(Long idDepartment) {
        return getAnnouncementsByDepartment(idDepartment, null, null);
    }

    public List<Announcement> getAnnouncementsByDepartment(Long idDepartment, Integer page, Integer pageSize) {
        if (page == null) page = 0;
        if (pageSize == null) pageSize = databaseConstants.defaultPageSize;
        return entityManager
                .createQuery("select a from Announcement a where a.department.id=:idDepartment", Announcement.class)
                .setParameter("idDepartment", idDepartment)
                .setFirstResult(page * pageSize)
                .setMaxResults(pageSize)
                .getResultList();
    }

    public Announcement getAnnouncementById(Long id) {
        Announcement announcement = announcementRepository.getById(id);
        List<Attachment> attachments = announcement.getAttachments()
                .parallelStream()
                .peek(attachment -> {
                    String contentAsBase64 = null;
                    try {
                        contentAsBase64 = s3Client.getAttachmentAsBase64(attachment.getFileName());
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                    attachment.setContentAsBase64(contentAsBase64);
                })
                .collect(Collectors.toList());

        announcement.setAttachments(attachments);
        return announcement;
    }

    private String generateUniqueAttachmentFilename(String filename) {
        return "anexo-" + UUID.randomUUID() + "-" + filename;
    }

}
